searchData={"items":[{"type":"module","title":"Once","doc":"Once is an Ecto type for locally unique 64-bits IDs generated by multiple Elixir nodes. Locally unique IDs make it easier to keep things separated, simplify caching and simplify inserting related things (because you don't have to wait for the database to return the ID). A Once can be generated in multiple ways:\n\n- counter (default): really fast to generate, predictable, works well with b-tree indexes\n- encrypted: unique and unpredictable, like a UUIDv4 but shorter\n- sortable: time-sortable like a [Snowflake ID](https://en.wikipedia.org/wiki/Snowflake_ID)\n\nA Once can look however you want, and can be stored in multiple ways as well. By default, in Elixir it's a url64-encoded 11-char string, and in the database it's a signed bigint. By using the `:ex_format` and `:db_format` options, you can choose both the Elixir and storage format out of `t:format/0`. You can pick any combination and use `to_format/2` to transform them as you wish!\n\nBecause a Once fits into an SQL bigint, they use little space and keep indexes small and fast. Because of their [structure](https://hexdocs.pm/no_noncense/NoNoncense.html#module-how-it-works) they have counter-like data locality, which helps your indexes perform well, [unlike UUIDv4s](https://www.cybertec-postgresql.com/en/unexpected-downsides-of-uuid-keys-in-postgresql/). If you don't care about that and want unpredictable IDs, you can use encrypted IDs that seem random and are still unique.\n\nThe actual values are generated by `NoNoncense`, which performs incredibly well, hitting rates of tens of millions of nonces per second, and it also helps you to safeguard the uniqueness guarantees.\n\nThe library has only `Ecto` and its sibling `NoNoncense` as dependencies.","ref":"Once.html"},{"type":"module","title":"Usage - Once","doc":"To get going, you need to set up a `NoNoncense` instance to generate the base unique values. Follow [its documentation](https://hexdocs.pm/no_noncense) to do so. `Once` expects an instance with its own module name by default, like so:\n\n    # application.ex (read the NoNoncense docs!)\n    machine_id = NoNoncense.MachineId.id!(opts)\n    NoNoncense.init(name: Once, machine_id: machine_id)\n\nIn your `Ecto` schemas, you can then use the type:\n\n    schema \"things\" do\n      field :id, Once\n    end\n\nAnd that's it!","ref":"Once.html#module-usage"},{"type":"module","title":"Options - Once","doc":"The Ecto type takes a few optional parameters:\n\n- `:no_noncense` name of the NoNoncense instance used to generate new IDs (default `Once`)\n- `:ex_format` what an ID looks like in Elixir, one of `t:format/0` (default `:url64`)\n- `:db_format` what an ID looks like in your database, one of `t:format/0` (default `:signed`)\n- `:type` how the nonce is generated, one of `t:nonce_type/0` (default `:counter`)\n- `:get_key` a zero-arity getter for the 192-bits encryption key, required if encryption is enabled\n- `:encrypt?` **deprecated**, use `type: :encrypted` (default `false`).","ref":"Once.html#module-options"},{"type":"module","title":"Data formats - Once","doc":"There's a drawback to having different data formats for Elixir and SQL: it makes it harder to compare the two. The following are all the same ID:\n\n    -1\n    <<255, 255, 255, 255, 255, 255, 255, 255>>\n    \"__________8\"\n    18_446_744_073_709_551_615\n    \"FFFFFFFFFFFFFFFF\"\n\nIf you use the defaults `:url64` as the Elixir format and `:signed` in your database, you could see `\"AAAAAACYloA\"` in Elixir and `10_000_000` in your database. The reasoning behind these defaults is that the encoded format is readable, short, and JSON safe by default, while the signed format means you can use a standard bigint column type.\n\nThe negative integers will not cause problems with Postgres and MySQL, they both happily swallow them. Also, negative integers will only start to appear after ~70 years of usage.\n\nIf you don't like the formats, it's really easy to change them! The Elixir format especially, which can be changed at any time. Be mindful of JSON limitations if you use integers.\n\nThe supported formats are:\n\n- `:url64` a url64-encoded string of 11 characters, for example `\"AAjhfZyAAAE\"`\n- `:hex` a hex-encoded string of 16 characters, for example `\"E010831058218A39\"`\n- `:raw` a bitstring of 64 bits, for example `<<0, 8, 225, 125, 156, 128, 0, 2>>`\n- `:signed` a signed 64-bits integer, like `-12345`, between -(2^63) and 2^63-1\n- `:unsigned` an unsigned 64-bits integer, like `67890`, between 0 and 2^64-1","ref":"Once.html#module-data-formats"},{"type":"module","title":"On local uniqueness - Once","doc":"By locally unique, we mean unique within your domain or application. UUIDs are globally unique across domains, servers and applications. A Once is not, because 64 bits is not enough to achieve that. It is enough for local uniqueness however: you can generate 8 million IDs per second on 512 machines in parallel for 140 years straight before you run out of bits, by which time your great-grandchildren will deal with the problem. Even higher burst rates are possible and you can use separate `NoNoncense` instanses for every table if you wish.","ref":"Once.html#module-on-local-uniqueness"},{"type":"module","title":"Encrypted IDs - Once","doc":"By default, IDs are generated using a machine init timestamp, machine ID and counter (although they should be considered to be opague). This means they leak a little information and are somewhat predictable. If you don't like that, you can use encrypted IDs by passing options `type: :encrypted` and `get_key: fn -> <<_::192>> end`. Note that encrypted IDs will cost you the data locality and decrease index performance a little. The encryption algorithm is 3DES and that can't be changed. If you want to know why, take a look at [NoNoncense](https://hexdocs.pm/no_noncense/NoNoncense.html#module-encrypted-nonces).","ref":"Once.html#module-encrypted-ids"},{"type":"type","title":"Once.format/0","doc":"Formats in which a `Once` can be rendered.\nThey are all equivalent and can be transformed to one another.\n\n- `:url64` a url64-encoded string of 11 characters, for example `\"AAjhfZyAAAE\"`\n- `:hex` a hex-encoded string of 16 characters, for example `\"E010831058218A39\"`\n- `:raw` a bitstring of 64 bits, for example `<<0, 8, 225, 125, 156, 128, 0, 2>>`\n- `:signed` a signed 64-bits integer, like `-12345`, between -(2^63) and 2^63-1\n- `:unsigned` an unsigned 64-bits integer, like `67890`, between 0 and 2^64-1","ref":"Once.html#t:format/0"},{"type":"type","title":"Once.nonce_type/0","doc":"The way in which the underlying 64-bits nonce is generated.\n\nSee `NoNoncense` for details.","ref":"Once.html#t:nonce_type/0"},{"type":"type","title":"Once.opts/0","doc":"Options to initialize `Once`.\n\n- `:no_noncense` name of the NoNoncense instance used to generate new IDs (default `Once`)\n- `:ex_format` what an ID looks like in Elixir, one of `t:format/0` (default `:url64`)\n- `:db_format` what an ID looks like in your database, one of `t:format/0` (default `:signed`)\n- `:type` how the nonce is generated, one of `t:nonce_type/0` (default `:counter`)\n- `:get_key` a zero-arity getter for the 192-bits encryption key, required if encryption is enabled\n- `:encrypt?` **deprecated**, use `type: :encrypted` (default `false`).","ref":"Once.html#t:opts/0"},{"type":"function","title":"Once.to_format/2","doc":"Transform the different forms that a `Once` can take to one another.\nThe formats can be found in `t:format/0`.\n\n    iex> Once.to_format(\"4BCDEFghijk\", :raw)\n    {:ok, <<224, 16, 131, 16, 88, 33, 138, 57>>}\n    iex> Once.to_format(<<224, 16, 131, 16, 88, 33, 138, 57>>, :signed)\n    {:ok, -2301195303365014983}\n    iex> Once.to_format(-2301195303365014983, :unsigned)\n    {:ok, 16145548770344536633}\n    iex> Once.to_format(16145548770344536633, :hex)\n    {:ok, \"E010831058218A39\"}\n    iex> Once.to_format(\"E010831058218a39\", :url64)\n    {:ok, \"4BCDEFghijk\"}\n\n    iex> Once.to_format(-1, :url64)\n    {:ok, \"__________8\"}\n    iex> Once.to_format(\"__________8\", :raw)\n    {:ok, <<255, 255, 255, 255, 255, 255, 255, 255>>}\n    iex> Once.to_format(<<255, 255, 255, 255, 255, 255, 255, 255>>, :unsigned)\n    {:ok, 18446744073709551615}\n    iex> Once.to_format(18446744073709551615, :hex)\n    {:ok, \"FFFFFFFFFFFFFFFF\"}\n    iex> Once.to_format(\"FFFFFFFFFFFFFFFF\", :signed)\n    {:ok, -1}\n\n    iex> Once.to_format(Integer.pow(2, 64), :unsigned)\n    :error","ref":"Once.html#to_format/2"},{"type":"function","title":"Once.to_format!/2","doc":"Same as `to_format/2` but raises on error.\n\n    iex> -200\n    ...> |> Once.to_format!(:url64)\n    ...> |> Once.to_format!(:raw)\n    ...> |> Once.to_format!(:unsigned)\n    ...> |> Once.to_format!(:hex)\n    ...> |> Once.to_format!(:signed)\n    -200\n\n    iex> Once.to_format!(Integer.pow(2, 64), :unsigned)\n    ** (ArgumentError) value could not be parsed","ref":"Once.html#to_format!/2"},{"type":"extras","title":"Once","doc":"# Once\n\nOnce is an Ecto type for locally unique 64-bits IDs generated by multiple Elixir nodes. Locally unique IDs make it easier to keep things separated, simplify caching and simplify inserting related things (because you don't have to wait for the database to return the ID). A Once can be generated in multiple ways:\n\n- counter (default): really fast to generate, predictable, works well with b-tree indexes\n- encrypted: unique and unpredictable, like a UUIDv4\n- sortable: time-sortable like a [Snowflake ID](https://en.wikipedia.org/wiki/Snowflake_ID)\n\nA Once can look however you want, and can be stored in multiple ways as well. By default, in Elixir it's a url64-encoded 11-char string, and in the database it's a signed bigint. By using the `:ex_format` and `:db_format` options, you can choose both the Elixir and storage format out of `t:format/0`. You can pick any combination and use `to_format/2` to transform them as you wish!\n\nBecause a Once fits into an SQL bigint, they use little space and keep indexes small and fast. Because of their [structure](https://hexdocs.pm/no_noncense/NoNoncense.html#module-how-it-works) they have counter-like data locality, which helps your indexes perform well, [unlike UUIDv4s](https://www.cybertec-postgresql.com/en/unexpected-downsides-of-uuid-keys-in-postgresql/). If you don't care about that and want unpredictable IDs, you can use encrypted IDs that seem random and are still unique.\n\nThe actual values are generated by `NoNoncense`, which performs incredibly well, hitting rates of tens of millions of nonces per second, and it also helps you to safeguard the uniqueness guarantees.\n\nThe library has only `Ecto` and its sibling `NoNoncense` as dependencies.","ref":"readme.html"},{"type":"extras","title":"Installation - Once","doc":"The package is hosted on [hex.pm](https://hex.pm/packages/once) and can be installed by adding `:once` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:once, \"~> 0.0.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Docs - Once","doc":"Documentation can be found on [hexdocs.pm](https://hexdocs.pm/once/).","ref":"readme.html#docs"},{"type":"extras","title":"Usage - Once","doc":"To get going, you need to set up a `NoNoncense` instance to generate the base unique values. Follow [its documentation](https://hexdocs.pm/no_noncense) to do so. `Once` expects an instance with its own module name by default, like so:\n\n```elixir\n# application.ex (read the NoNoncense docs!)\nmachine_id = NoNoncense.MachineId.id!(opts)\nNoNoncense.init(name: Once, machine_id: machine_id)\n```\n\nIn your `Ecto` schemas, you can then use the type:\n\n```elixir\nschema \"things\" do\n  field :id, Once\nend\n```\n\nAnd that's it! Be sure to look at [hexdocs.pm](https://hexdocs.pm/once/) for options and additional info.","ref":"readme.html#usage"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}